#include "StdAfx.h"
#include "DynamicPopupMenu.h"


CDynamicPopupMenu::~CDynamicPopupMenu(void)
{
	m_menuMap.clear();
}

void CDynamicPopupMenu::AddItemData(const int item_id, const int parent_id, bool is_visible, bool check_flag, 
												bool has_child, const std::wstring item_name, bool enable_flag)
{
	ItemsData	ob;

	ob.nID				= item_id;
	ob.nParentID		= parent_id;
	ob.bIsVisible		= is_visible;
	ob.bCheckFlag		= check_flag;
	ob.bHasChilds		= has_child;	
	ob.sItemName		= item_name;
	ob.bEnableFlag		= enable_flag;
	ob.bIsSeparator		= false;

	m_menuMap.insert(std::pair<int, ItemsData>(item_id, ob));
}

void CDynamicPopupMenu::AddItemSeparator(const int item_id, const int parent_id)
{	
	ItemsData	ob;

	ob.nID				= item_id;
	ob.nParentID		= parent_id;
	ob.bIsSeparator		= true;

	m_menuMap.insert(std::pair<int, ItemsData>(item_id, ob));
}

bool CDynamicPopupMenu::SetEnableFlagItem(const int nMapKey, bool bFlag)
{

	iterDynMenu it = m_menuMap.find(nMapKey);

	if (it == m_menuMap.end())
		return false;

	it->second.bEnableFlag = bFlag;

	return true;
}

bool CDynamicPopupMenu::GetEnableFlagItem(const int nMapKey, bool& bFlag)
{
	bool bRet = false;

	iterDynMenu it = m_menuMap.find(nMapKey);
	
	if (it == m_menuMap.end())
		return false;
	
	bFlag = it->second.bEnableFlag;

	return true;
}

bool CDynamicPopupMenu::SetVisibleFlagItem(const int nMapKey, bool bFlag)
{
	iterDynMenu it = m_menuMap.find(nMapKey);	
	
	if (it == m_menuMap.end())
		return false;

	it->second.bIsVisible = bFlag;

	return true;
}

bool CDynamicPopupMenu::GetVisibleFlagItem(const int nMapKey, bool& bFlag)
{
	iterDynMenu it = m_menuMap.find(nMapKey);

	if (it == m_menuMap.end())
		return false;

	bFlag = it->second.bIsVisible;

	return true;
}

bool CDynamicPopupMenu::SetCheckedItemFlag(const int nMapKey,const bool bFlag)
{
	bool bRet = false;

	if (!m_menuMap.empty())
	{
		if (0 == m_menuMap[nMapKey].nParentID)
		{
			m_menuMap[nMapKey].bCheckFlag = bFlag;
		}	
		else
		{
			for (iterDynMenu itMap = m_menuMap.begin();	itMap != m_menuMap.end(); ++itMap)
			{
				if (itMap->first  == nMapKey)
				{
					itMap->second.bCheckFlag = bFlag;
					int nParentID = m_menuMap[nMapKey].nParentID;

					if (bFlag)
					{
						m_menuMap[nParentID].bCheckFlag = bFlag;
					}
					else		
					{
						bool bShowParent = false;
						for (iterDynMenu itM = m_menuMap.begin();	itM != m_menuMap.end(); ++itM)
						{
							// check if this parent item has other active child
							if ((nParentID == itM->second.nParentID) 
										&& (true == itM->second.bCheckFlag))
							{
								bShowParent = true;
							}
						}

						m_menuMap[nParentID].bCheckFlag = bShowParent;
					}		

					bRet = true;
					break;
				}
			}
		}
	}
	
	return bRet;
}

bool CDynamicPopupMenu::GetItemCheckedFlag(const int nMapKey, bool& bFlag)
{
	iterDynMenu itMap = m_menuMap.find(nMapKey);

	if (itMap == m_menuMap.end())
		return false;
	
	bFlag = itMap->second.bCheckFlag;

	return true;
}

DWORD CDynamicPopupMenu::TrackPopupCustomMenu(POINT point, HWND hWnd)
{
	std::vector<HMENU>				vSubMenu;
	std::vector<HMENU>::iterator	itSM;

	DWORD	dwRet = 0;
	HMENU	m_hPopupMenu = CreatePopupMenu();

	for (iterDynMenu it = m_menuMap.begin(); it != m_menuMap.end(); ++it)
	{
		if ((0 == it->second.nParentID) && (!it->second.bHasChilds))
		{
			if (it->second.bIsSeparator)
				AppendMenu(m_hPopupMenu, MF_SEPARATOR, 0 , _T(""));
			else
			{
				UINT nFlags = MF_STRING;
				nFlags |= (it->second.bEnableFlag) ? MF_ENABLED : MF_GRAYED;

				AppendMenu(m_hPopupMenu, nFlags, it->second.nID, it->second.sItemName.c_str());
				CheckMenuItem(m_hPopupMenu, it->first, it->second.bCheckFlag ? MF_CHECKED | MF_BYCOMMAND : MF_UNCHECKED | MF_BYCOMMAND);
			}
		}
		else
			if ((0 == it->second.nParentID) && (true == it->second.bHasChilds))			// find a child
			{
				bool isFirstSubItem = true;

				HMENU hSubMenu = CreatePopupMenu();
				vSubMenu.push_back(hSubMenu);

				itSM = vSubMenu.end();
				--itSM;

				for (iterDynMenu it_child = m_menuMap.begin(); it_child != m_menuMap.end(); ++it_child)
				{
					if (it->first == it_child->second.nParentID)		// this is a child
					{
						if (isFirstSubItem)		
						{
							if	(it_child->second.bIsSeparator)
							{
								AppendMenu(*itSM, MF_SEPARATOR, 0, _T(""));
							}
							else
							{
								UINT nFlags = MF_POPUP | MF_BYCOMMAND;
								nFlags |= (it->second.bEnableFlag) ? MF_ENABLED : MF_GRAYED;

								InsertMenu(m_hPopupMenu, it->first, nFlags,  (UINT_PTR)hSubMenu, it->second.sItemName.c_str());
								CheckMenuItem(m_hPopupMenu, it->first, 
										it->second.bCheckFlag ? MF_CHECKED | MF_BYCOMMAND : MF_UNCHECKED | MF_BYCOMMAND);

								AppendMenu(*itSM, MF_STRING, it_child->second.nID, it_child->second.sItemName.c_str());
								CheckMenuItem(m_hPopupMenu, it_child->first, 
											it_child->second.bCheckFlag ? MF_CHECKED | MF_BYCOMMAND : MF_UNCHECKED | MF_BYCOMMAND);
						
								EnableMenuItem(*itSM, it_child->first,
												it_child->second.bEnableFlag ? MF_ENABLED | MF_BYCOMMAND : MF_GRAYED | MF_BYCOMMAND);
							}

							isFirstSubItem = false;
						}
						else
						{
							// other sub-items
							if (it_child->second.bIsSeparator)
								AppendMenu(*itSM, MF_SEPARATOR,	0, _T(""));
							else
							{
								AppendMenu(*itSM, MF_STRING, it_child->second.nID, it_child->second.sItemName.c_str());
								CheckMenuItem(m_hPopupMenu, it_child->first, 
										it_child->second.bCheckFlag ? MF_CHECKED | MF_BYCOMMAND : MF_UNCHECKED | MF_BYCOMMAND);

								EnableMenuItem(*itSM, it_child->first,
										it_child->second.bEnableFlag ? MF_ENABLED | MF_BYCOMMAND : MF_GRAYED | MF_DISABLED | MF_BYCOMMAND);
							}
						}
					}
				}
			}
	}

	dwRet =	 TrackPopupMenuEx(m_hPopupMenu, TPM_NONOTIFY | TPM_RETURNCMD, point.x,point.y,  hWnd, NULL );

	for (itSM = vSubMenu.begin(); itSM != vSubMenu.end(); ++itSM)
	{
		DestroyMenu(*itSM);
	}

	DestroyMenu(m_hPopupMenu);

	return dwRet;
}

void CDynamicPopupMenu::SelectShowAllGroupItems(const int nKeyShowAll, bool bIsVisible)
{
	iterDynMenu itMap;
	itMap = m_menuMap.find(nKeyShowAll);
	if (itMap == m_menuMap.end())
		return;

	int nParentID =  itMap->second.nParentID;

	if (0 != nParentID)		
		itMap->second.bIsVisible = bIsVisible;			// checked/unchecked parent item

	for (iterDynMenu itMap = m_menuMap.begin(); itMap != m_menuMap.end(); ++itMap)
	{
		if ( (nParentID == itMap->second.nParentID) && (!itMap->second.bIsSeparator) 
					&& (nKeyShowAll != itMap->first) && (itMap->second.bEnableFlag) )		
		{
			itMap->second.bIsVisible = bIsVisible;
		}
	}
}

void CDynamicPopupMenu::CheckAllGroupItems(const int nKeyCheckAll, bool bChecked)
{
	iterDynMenu itMap;
	itMap = m_menuMap.find(nKeyCheckAll);
	if (itMap == m_menuMap.end())
		return;

	int nParentID = itMap->second.nParentID;

	if (0 != nParentID)		
		itMap->second.bCheckFlag = bChecked;			// checked/unchecked parent item

	for (iterDynMenu itMap = m_menuMap.begin(); itMap != m_menuMap.end(); ++itMap)
	{
		if ( (nParentID == itMap->second.nParentID) && (!itMap->second.bIsSeparator) 
								&& (itMap->second.bEnableFlag) )		
		{
			itMap->second.bCheckFlag = bChecked;
		}
	}
}